from .spanish_news_classification import (
    SpanishNewsClassification,
    SpanishNewsClassificationV2,
)
from .spanish_sentiment_classification import (
    SpanishSentimentClassification,
    SpanishSentimentClassificationV2,
)

__all__ = [
    "SpanishNewsClassification",
    "SpanishNewsClassificationV2",
    "SpanishSentimentClassification",
    "SpanishSentimentClassificationV2",
]
