from .biosses_sts import BiossesSTS
from .humesickr import HUMESICKR
from .humests12 import HUMESTS12
from .humests_benchmark import HUMESTSBenchmark
from .sickr_sts import SickrSTS
from .sts12_sts import STS12STS
from .sts12_visual_sts import STS12VisualSTS
from .sts13_sts import STS13STS
from .sts13_visual_sts import STS13VisualSTS
from .sts14_sts import STS14STS
from .sts14_visual_sts import STS14VisualSTS
from .sts15_sts import STS15STS
from .sts15_visual_sts import STS15VisualSTS
from .sts16_sts import STS16STS
from .sts16_visual_sts import STS16VisualSTS
from .sts_benchmark_sts import STSBenchmarkSTS

__all__ = [
    "HUMESICKR",
    "HUMESTS12",
    "STS12STS",
    "STS13STS",
    "STS14STS",
    "STS15STS",
    "STS16STS",
    "BiossesSTS",
    "HUMESTSBenchmark",
    "STS12VisualSTS",
    "STS13VisualSTS",
    "STS14VisualSTS",
    "STS15VisualSTS",
    "STS16VisualSTS",
    "STSBenchmarkSTS",
    "SickrSTS",
]
