from .sick_nl_pair_classification import SICKNLPairClassification
from .xlwic_nl_pair_classification import XLWICNLPairClassification

__all__ = [
    "SICKNLPairClassification",
    "XLWICNLPairClassification",
]
